@@ -2,8 +2,11 @@ import os
 import subprocess as sp
 import unittest
 import numpy as np
+import pysam
 import pytest
 import conftest
+import pandas as pd
+import math
 
 try:
     import scipy
@@ -80,8 +83,9 @@ class HTSeqCountBase(unittest.TestCase):
         else:
             raise ValueError(f'Format not supported: {fmt}')
 
-    def _run(self, t):
+    def _run(self, t, remove_res_files=True):
         expected_fn = t.get('expected_fn', None)
+        expected_stderr = t.get('expected_stderr', None)
         call = t['call']
 
         # Replace with injected variable
@@ -96,10 +100,16 @@ class HTSeqCountBase(unittest.TestCase):
         #    call = ['python', 'HTSeq/scripts/count_with_barcodes.py'] + call[1:]
 
         print(' '.join(call))
-        output = sp.check_output(
-                ' '.join(call),
-                shell=True,
-        ).decode()
+        p = sp.run(
+            ' '.join(call),
+            shell=True,
+            check=True,
+            capture_output=True,
+        )
+        if expected_stderr:
+            actual_stderr_str = p.stderr.decode()
+            self.assertEqual(actual_stderr_str, expected_stderr)
+        output = p.stdout.decode()
 
         if '-c' in call:
             output_fn = call[call.index('-c') + 1]
@@ -135,9 +145,10 @@ class HTSeqCountBase(unittest.TestCase):
                 close_file(output_fn, output)
                 close_file(expected_fn, expected)
                 # FIXME
-                if True:#output['fmt'] not in ['h5ad', 'loom']:
+                if remove_res_files:#output['fmt'] not in ['h5ad', 'loom']:
                     os.remove(output_fn)
 
+
 class HTSeqCount(HTSeqCountBase):
     cmd = 'htseq-count'
 
@@ -205,6 +216,87 @@ class HTSeqCount(HTSeqCountBase):
             'expected_fn': f'{data_folder}/bamfile_no_qualities.loom',
             })
 
+    def test_output_tsv_header(self):
+        # Header must be there
+        self._run({
+            'call': [
+                self.cmd,
+                '--with-header', ' ',
+                '-c', f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell1.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell2.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell3.bam',
+                f'{data_folder}/10x_pbmc1k/HomoSapiens.GRCh38-2020-A_subsampled.gtf',
+            ],
+            'expected_fn': f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+        }, remove_res_files=False)
+
+        try:
+            dat = pd.read_csv(f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv', delimiter='\t')
+            self.assertEqual(len(dat.columns), 4)
+            self.assertTrue(f'{data_folder}10x_pbmc1k/3_cells/cell1.bam' in dat.columns)
+            self.assertTrue(f'{data_folder}10x_pbmc1k/3_cells/cell2.bam' in dat.columns)
+            self.assertTrue(f'{data_folder}10x_pbmc1k/3_cells/cell3.bam' in dat.columns)
+        finally:
+            os.remove(f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv')
+
+    def test_output_tsv_no_header(self):
+        # Backwards compatibility of no header
+        self._run({
+            'call': [
+                self.cmd,
+                '-c', f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell1.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell2.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell2.bam',
+                f'{data_folder}/10x_pbmc1k/HomoSapiens.GRCh38-2020-A_subsampled.gtf',
+            ],
+            'expected_fn': f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+        }, remove_res_files=False)
+        try:
+            dat = pd.read_csv(f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv', delimiter='\t', header=None)
+            self.assertEqual(dat.shape[0], 7)
+            self.assertEqual(dat.loc[0,0], 'ENSG00000188976')
+        finally:
+            os.remove(f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv')
+
+    def test_output_tsv_header_and_append(self):
+        # Make sure append still works properly
+        self._run({
+            'call': [
+                self.cmd,
+                '--with-header', ' ',
+                '-c', f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell1.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell2.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell3.bam',
+                f'{data_folder}/10x_pbmc1k/HomoSapiens.GRCh38-2020-A_subsampled.gtf',
+            ],
+            'expected_fn': f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+        }, remove_res_files=False)
+
+        self._run({
+            'call': [
+                self.cmd,
+                '--append-output', ' ',
+                '-c', f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell1.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell2.bam',
+                f'{data_folder}/10x_pbmc1k/3_cells/cell3.bam',
+                f'{data_folder}/10x_pbmc1k/HomoSapiens.GRCh38-2020-A_subsampled.gtf',
+            ],
+            'expected_fn': f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv',
+        }, remove_res_files=False)
+        try:
+            dat = pd.read_csv(f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv', delimiter='\t')
+            self.assertEqual(dat.shape[0], 14)
+            self.assertTrue(f'{data_folder}10x_pbmc1k/3_cells/cell1.bam' in dat.columns)
+            self.assertTrue(f'{data_folder}10x_pbmc1k/3_cells/cell2.bam' in dat.columns)
+            self.assertTrue(f'{data_folder}10x_pbmc1k/3_cells/cell3.bam' in dat.columns)
+        finally:
+            os.remove(f'{data_folder}/10x_pbmc1k/test_tsv_header.tsv')
+
+
     # Testing multiple cores on travis makes a mess
     #{'call': [
     #    'htseq-count',
@@ -316,6 +408,43 @@ class HTSeqCount(HTSeqCountBase):
             'expected_fn': f'{data_folder}/yeast_RNASeq_excerpt_withNH_counts_additional_attributes_chromosome_info.tsv',
             })
 
+    @unittest.skipIf(anndata is None, "test case depends on anndata")
+    def test_additional_attributes_h5ad(self):
+        # Get gene name as additional attr but output as h5ad file.
+        self._run({
+            'call': [
+                self.cmd,
+                '-m', 'intersection-nonempty',
+                '--nonunique', 'none',
+                '--secondary-alignments', 'score',
+                '--supplementary-alignments', 'score',
+                '--additional-attr', 'gene_name',
+                '--additional-attr', 'exon_number',
+                '-c', f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes_counts.h5ad',
+                f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes.sam',
+                f'{data_folder}/10x_pbmc1k/HomoSapiens.GRCh38-2020-A_subsampled.gtf',
+                ],
+            'expected_fn': f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes_counts.h5ad',
+            }, remove_res_files=False)
+
+        # Check gene name and exon number is there
+        try:
+            dat = anndata.read_h5ad(f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes_counts.h5ad')
+            self.assertEqual(dat.var.loc['ENSG00000188976', 'gene_name'], 'NOC2L')
+            self.assertEqual(dat.var.loc['ENSG00000251562', 'gene_name'], 'MALAT1')
+            self.assertEqual(dat.var.loc['ENSG00000188976', 'exon_number'], '2')
+            self.assertEqual(dat.var.loc['ENSG00000251562', 'exon_number'], '2')
+
+            htseq_specific_features = ['__no_feature', '__ambiguous', '__too_low_aQual',
+                                       '__not_aligned', '__alignment_not_unique']
+            for feat in htseq_specific_features:
+                self.assertTrue(math.isnan(dat.var.loc[feat, 'gene_name']))
+                self.assertTrue(math.isnan(dat.var.loc[feat, 'exon_number']))
+
+        finally:
+            # clean up
+            os.remove(data_folder + "/10x_pbmc1k/subsampled_with_missing_barcodes_counts.h5ad")
+
     def test_nonunique_fraction(self):
         self._run({
             'call': [
@@ -375,6 +504,39 @@ class HTSeqCount(HTSeqCountBase):
             'expected_fn': f'{data_folder}/yeast_RNASeq_excerpt_withNH_counts_ignore_secondary.tsv',
             })
 
+    def test_no_contig_overlap_warning(self):
+        """ Ensures that no contig warning is present in stderr
+            @see https://github.com/htseq/htseq/issues/63 """
+
+        # We print full file path in the error message, so have to use template & replace
+        bam_file = os.path.join(data_folder, 'SRR001432_head_sorted.bam')
+        stderr_template = open(f'{data_folder}/no_contig_overlap_warning_stderr.txt').read()
+        expected_stderr = stderr_template.format(FILENAME=bam_file)
+        self._run({
+            'call': [
+                self.cmd,
+                bam_file,
+                f'{data_folder}/bamfile_no_qualities.gtf',
+                ],
+                'expected_stderr': expected_stderr,
+            })
+
+    def test_contig_overlap_no_warning(self):
+        """ Ensures that warning is NOT present in stderr when there are contig overlaps
+            @see https://github.com/htseq/htseq/issues/63 """
+
+        bam_file = os.path.join(data_folder, 'bamfile_no_qualities.sam')
+        expected_stderr = open(f'{data_folder}/contig_overlap_no_warning_stderr.txt').read()
+        self._run({
+            'call': [
+                self.cmd,
+                bam_file,
+                f'{data_folder}/bamfile_no_qualities.gtf',
+                ],
+                'expected_stderr': expected_stderr,
+            })
+
+
 
 class HTSeqCountBarcodes(HTSeqCountBase):
     cmd = 'htseq-count-barcodes'
@@ -385,6 +547,7 @@ class HTSeqCountBarcodes(HTSeqCountBase):
                 self.cmd,
                 '--version'],
             })
+
     def test_simple(self):
         self._run({
             'call': [
@@ -478,6 +641,67 @@ class HTSeqCountBarcodes(HTSeqCountBase):
             'expected_fn': f'{data_folder}/yeast_RNASeq_excerpt_withbarcodes.loom',
             })
 
+    def test_missing_barcodes(self):
+        """
+        When reads are missing cell or UMI barcodes, the counts should exclude these reads
+        """
+        self._run({
+            'call': [
+                self.cmd,
+                '--mode', 'union',
+                '--secondary-alignments', 'ignore',
+                '--supplementary-alignments', 'ignore',
+                '--stranded', 'yes',
+                '--counts_output_sparse',
+                '-c', f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes_counts.csv',
+                '--samout', f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes_counts.sam',
+                f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes.sam',
+                f'{data_folder}/10x_pbmc1k/HomoSapiens.GRCh38-2020-A_subsampled.gtf',
+                ],
+            'expected_fn': f'{data_folder}/10x_pbmc1k/subsampled_with_missing_barcodes_counts.csv',
+            }, remove_res_files = False)
+
+        # Check the result. The count csv should not register any reads with low quality
+        # The SAM file should assign low quality to reads without cell or UMI barcode
+
+        samfile = None
+        try:
+            df = pd.read_csv(data_folder + '/10x_pbmc1k/subsampled_with_missing_barcodes_counts.csv',
+                             header=None,
+                             delimiter='\t')
+            df.columns = ['gene_ids', 'count']
+            self.assertEqual(int(df.loc[df['gene_ids'] == 'ENSG00000188976']['count']), 1)
+            self.assertEqual(int(df.loc[df['gene_ids'] == 'ENSG00000251562']['count']), 1)
+
+            other_genes = ['__no_feature', '__ambiguous', '__too_low_aQual', '__not_aligned', '__alignment_not_unique']
+
+            for g in other_genes:
+                self.assertEqual(int(df.loc[df['gene_ids'] == g]['count']), 0)
+
+            samfile = pysam.AlignmentFile(data_folder + "/10x_pbmc1k/subsampled_with_missing_barcodes_counts.sam")
+            read_assignments = {
+                "A00228:279:HFWFVDMXX:2:1385:2085:18975": "__too_low_aQual",
+                "A00228:279:HFWFVDMXX:1:2158:25464:1313": "ENSG00000251562",
+                "A00228:279:HFWFVDMXX:2:1168:11153:29168": "ENSG00000188976",
+                "A00228:279:HFWFVDMXX:1:1425:2781:12665": "__too_low_aQual",
+                "A00228:279:HFWFVDMXX:2:2322:8621:9157": "__too_low_aQual",
+                "A00228:279:HFWFVDMXX:1:2119:24270:19351": "__too_low_aQual",
+                "A00228:279:HFWFVDMXX:2:2249:1045:2707": "__too_low_aQual",
+                "A00228:279:HFWFVDMXX:2:2401:14172:19210": "__too_low_aQual"
+            }
+
+            for read in samfile.fetch(until_eof=True):
+                qname_str = read.qname
+                tag_val = read.get_tag("XF")
+                self.assertEqual(tag_val, read_assignments[qname_str])
+
+        finally:
+            # clean up
+            if samfile is not None:
+                samfile.close()
+            os.remove(data_folder + '/10x_pbmc1k/subsampled_with_missing_barcodes_counts.csv')
+            os.remove(data_folder + "/10x_pbmc1k/subsampled_with_missing_barcodes_counts.sam")
+
 
 if __name__ == '__main__':
 
@@ -496,4 +720,4 @@ if __name__ == '__main__':
 
     suite = HTSeqCountBarcodes()
     suite.test_version()
-    suite.test_barcodes()
+    suite.test_missing_barcodes()
